

package SIMM

    model POSITIV
      Modelica.Blocks.Interfaces.RealInput u ;
      Modelica.Blocks.Math.Abs abs1 ;
      Modelica.Blocks.Math.Add add1 ;
      Modelica.Blocks.Math.Gain gain1(k = 1 / 2) ;
      Modelica.Blocks.Interfaces.RealOutput y ;
    equation
      connect(u,add1.u2) ;
      connect(add1.y,gain1.u) ;
      connect(gain1.y,y) ;
      connect(abs1.y,add1.u1) ;
      connect(u,abs1.u) ;
    end POSITIV;

    partial model PartialCompliant "Compliant connection of two translational 1D flanges"
      Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a "Left flange of compliant 1-dim. translational component";
      Modelica.Mechanics.Translational.Interfaces.Flange_b flange_b "Right flange of compliant 1-dim. translational component" ;
      Real s_rel(start = 0) "relative distance (= flange_b.s - flange_a.s)";
      Real f "force between flanges (positive in direction of flange axis R)";
    equation
      s_rel = flange_b.s - flange_a.s;
      flange_b.f = f;
      flange_a.f = -f;
    end PartialCompliant;

    package SIunits
        type AbsoluteActivity = Real ;
        type AbsolutePressure = Pressure ;
        type Pressure = Real ;
        type VolumeFlowRate = Real ;
        type Area = Real;
        type Length = Real ;
        type Velocity = Real ;
        type Volume = Real ;
        type Power = Real ;
        type AngularVelocity = Real ;
        type Efficiency = Real ;
        type Mass = Real ;
        type Force = Real ;
        type Torque = Real ;
        type Distance = Real;
           
    end SIunits;

      model RotationalFriction "Rotational Damping and Coulomb friction"
        extends Modelica.Mechanics.Rotational.Interfaces.TwoFlangesAndBearing;
        Real phi "Absolute angular position of flange_a and flange_b";
        Real w "Absolute angular velocity of flange_a and flange_b";
        Real a "Absolute angular acceleration of flange_a and flange_b";
        //Real tau ; 
        parameter Real Tau_prop = 1 "Angular velocity dependent friction";
        parameter Real Tau_Coulomb = 5 "Constant fricton: Coulomb torque";
        //Real epsilon = 0.001;
      equation
        phi = phi_a;
        phi = phi_b;
        w = der(phi);
        a = der(w);
        bearing.phi = 0;
        0 = flange_a.tau + flange_b.tau - Tau_prop * w - Tau_Coulomb*tanh(100*w) ;
        //tau = if w>=epsilon then Tau_Coulomb else -1*Tau_Coulomb ;
      end RotationalFriction;

end SIMM;


package Hydraulic "Librairie hydraulique élémentaire"
  package Connectors
    package Interfaces
    
      connector PortA "Layout of port where oil flows into an element"
        Real p "pressure at port";
        flow Real q "flow rate through port";
      end PortA;
      
      connector PortB "Hydraulic port where the fluid leaves the component"
        Real p "pressure at port";
        flow Real q "flow rate through port";
      end PortB;
      
      partial model pOnePort "Décrit un composant qui a un port hydraulique en entrée"
        PortA portA ;
      end pOnePort;
      
      model Chamber "Fluide dans une chambre avec un piston et une pression (sans compressibilite du fluide)"
        extends SIMM.PartialCompliant;
        parameter Real PistonArea  "Aire du piston";
        parameter Real RodLength  "Longueur de la tige";
        Real v_rel "Relative velocity between flange flange_a and flange_b";
        Real stroke "stroke of chamber";
        Real Volume "Volume = stroke * piston_area";
        PortA port_A "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
      equation
        stroke = s_rel - RodLength;
        Volume = stroke * PistonArea;
        port_A.q = -v_rel * PistonArea;
        flange_a.f = port_A.p * PistonArea;
        v_rel = der(s_rel);
      end Chamber;
      
      partial model TwoPortComponent "Superclass of circuits with two hydraulic ports"
        Real dp "pressure drop";
        Real q "flow rate through component";
        PortA portA "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
        PortB portB "Port B, were oil leaves the component (negative q, port_A.p > port_B.p means positive dp)" ;
      equation
        dp = portA.p - portB.p;
        q = portA.q;
        portA.q + portB.q = 0;
      end TwoPortComponent;
      
      partial model pTwoPortSystem "Superclass of circuits with two hydraulic ports"
        PortA port_A "Layout of port where oil flows into an element" ;
        PortB port_B "Hydraulic port where oil leaves the component" ;
       
      end pTwoPortSystem;
    end Interfaces;
    
    package Types
      type HydraulicCapacity = Real;
      type Conductance = Real;
    end Types;

    end Connectors;

 
  package Restrictions "Résistances hydrauliques"
    model Laminar "Resistance with laminar flow."
      extends Hydraulic.Connectors.Interfaces.TwoPortComponent;
      parameter Real G = 0.00000000000042 "conductance of laminar resistance";
    equation
      q = G * dp;
    end Laminar;
    
    model Turbulent "Resistance with turbulent flow."
      extends Hydraulic.Connectors.Interfaces.TwoPortComponent;
      parameter Real DPnom = 7000000.0 "Nominal difference of pressure (en Pa)";
      parameter Real Qnom = 1 "Nominal flow rate (en m^3/s)";
      parameter Real Pref = 0.000000001 "pressure ref for sign estimation";
    equation
      q = Qnom * sqrt(abs(dp) / DPnom) * tanh(dp / Pref);
    end Turbulent;
    
    model TurbulentControled "Controled resistance with turbulent flow."
      extends Hydraulic.Connectors.Interfaces.TwoPortComponent;
      parameter Real DPnom = 7000000.0 "Nominal difference of pressure (en Pa)";
      parameter Real Qnom = 1 "Nominal flow rate (en m^3/s)";
      parameter Real Imax = 1 "Maximal input";
    protected
      parameter Real Pref = 500000.0 "pressure ref for sign estimation and laminar approximation";
      parameter Real Iref = 0.000000001 "current ref for sign estimation";
    public
      Modelica.Blocks.Interfaces.RealInput i ;
    equation
      q = (Qnom * (tanh(i.signal / Iref) + 1)) / 2.0 * abs(i.signal / Imax) * sqrt(abs(dp) / DPnom) * tanh(dp / Pref);
    end TurbulentControled;
   
  end Restrictions;
  package Parts
    model RotComp "Perfect converter hydraulic-rotational"
      parameter Real Cyl = 1 "capacity volume (m^3/tr)";
      Real dp "pressure drop";
      Real q "débit dans le composant";
      Real w "angular velocity of pump shaft";
      Connectors.Interfaces.PortA portA "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
      Connectors.Interfaces.PortB portB "Port B, were oil leaves the component (negative q, port_A.p > port_B.p means positive dp)" ;
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange ;
    equation
      dp = portA.p - portB.p;
      q = portA.q;
      portA.q + portB.q = 0;
      w = der(flange.phi);
      // Pompe parfaite
      q = (Cyl * w) / 2.0 / Modelica.Constants.pi;
      dp = (flange.tau * 2.0 * Modelica.Constants.pi) / Cyl;
    end RotComp;
    
    model ChamberLeft "Incompressible fluid in a chamber with piston and pressure"
      parameter Real PistonArea  "Piston area";
      Real v_rel "Relative velocity between flange flange_a and flange_b";
      Real length "lenght of chamber";
      Real Volume "Volume = stroke * piston_area";
      Connectors.Interfaces.PortA port_A "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
      Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a ;
      Modelica.Mechanics.Translational.Interfaces.Flange_b flange_b ;
    equation
      // Efforts
      flange_a.f = flange_b.f;
      flange_a.f = port_A.p * PistonArea;
      // Deplacement
      Volume = length * PistonArea;
      port_A.q = -v_rel * PistonArea;
      v_rel = der(length);
      // Positions
      flange_b.s = flange_a.s + length;
    end ChamberLeft;
    model ChamberRight "Incompressible fluid in a chamber with piston and pressure"
      parameter Real PistonArea  "Aire du piston";
      Real v_rel "Relative velocity between flange flange_a and flange_b";
      Real length "lenght of chamber";
      Real Volume "Volume = stroke * piston_area";
      Connectors.Interfaces.PortA port_A "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
      Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a ;
      Modelica.Mechanics.Translational.Interfaces.Flange_b flange_b ;
    equation
      // Efforts
      flange_a.f = flange_b.f;
      flange_a.f = -port_A.p * PistonArea;
      // Deplacement
      Volume = length * PistonArea;
      port_A.q = v_rel * PistonArea;
      v_rel = der(length);
      // Positions
      flange_b.s = flange_a.s + length;
    end ChamberRight;
    
    model VolEfficiency "Leakage corresponding to volumetric efficiency"
      parameter Real Nuv = 0.95 "Rendement volumetrique";
      parameter Real Pref = 0.001 "Puissance de référence";
      Real qleak "débit de fuite";
      Real DP "Différence de pression";
      Connectors.Interfaces.PortA portA "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
      Connectors.Interfaces.PortB portB "Port B, were oil leaves the component (negative q, port_A.p > port_B.p means positive dp)" ;
      Connectors.Interfaces.PortB portC ;
    equation
      portA.p = portB.p;
      DP = portA.p - portC.p;
      qleak + portC.q = 0;
      portA.q + portB.q + portC.q = 0;
      qleak = ((1 - Nuv) * portA.q * (tanh((DP * portA.q) / Pref) + 1)) / 2.0 + ((1 - Nuv) * portB.q * (tanh((DP * portB.q) / Pref) + 1)) / 2.0;
    end VolEfficiency;
    
    model HydraulicCapacity "Capacité hydraulique"
      parameter Real Ch = 0.00001 "Hydraulic capacity (m^3/Pa)";
      Connectors.Interfaces.PortA port_A "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
    equation
      port_A.q = Ch * der(port_A.p);
    end HydraulicCapacity;

end Parts;

  package Components
    package Sources "Contient les sources hydrauliques"
      model FlowVolumetric "convertit un signal en débit (m^3/s)"
        Connectors.Interfaces.PortB port_B "débit en m^3/s" ;
        Modelica.Blocks.Interfaces.RealInput signal "Signal d'entrée" ;
      equation
        port_B.q = -signal.signal;
      end FlowVolumetric;
      
      model Pressure "convertit un signal en pression (Pa) or Bar"
        parameter Real isPascal = 1;
        Connectors.Interfaces.PortB port_B "pression en Pa" ;
        Modelica.Blocks.Interfaces.RealInput signal "Signal d'entrée" ;
      equation
        port_B.p = if isPascal > 0.0 then signal.signal else signal.signal / 100000.0;
      end Pressure;
      
    end Sources;
    
    package Sensors "Mesure des grandeurs hydraulique"
        model Pressure "Ideal pressure sensor"
            Connectors.Interfaces.PortA port_A ;
            Modelica.Blocks.Interfaces.RealOutput p;
        equation
            p.signal = port_A.p;
            0= port_A.q;
        end Pressure;
        model FlowVolumetricSensor "Ideal sensor for volume flow rate"
          Connectors.Interfaces.PortA port_A ;
          Connectors.Interfaces.PortA port_B ;
          Modelica.Blocks.Interfaces.RealOutput q "débit mesuré";
        equation
          port_A.p = port_B.p;
          port_A.q = q.signal;
          port_B.q =  -q.signal;
        end FlowVolumetricSensor;      
    end Sensors;
 
     model Cylinder "double acting cylinders"
      parameter Real Stroke = 0.05 "Course totale";
      parameter Real MobileMass  "Masse mobile (piston + tiges)";
      parameter Real PistonAreaLeft  "Effective area of left side of piston";
      parameter Real PistonAreaRight  "Effective area of right side of piston";
      Coselica.Mechanics.Translational.Components.Mass Piston(L = 0, m = MobileMass) ;
      Parts.ChamberLeft chamberPartialA(PistonArea = PistonAreaLeft) ;
      Parts.ChamberRight chamberPartialB(PistonArea = PistonAreaRight) ;
      Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a "Left flange" ;
      Modelica.Mechanics.Translational.Interfaces.Flange_b flange_b "Right flange" ;
      Modelica.Mechanics.Translational.Fixed FixedLeft(s0 = -Stroke / 2) ;
      Modelica.Mechanics.Translational.Fixed FixedRight(s0 = Stroke / 2) ;
      Connectors.Interfaces.PortA port_A1 "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)";
      Connectors.Interfaces.PortB port_A2 "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
    equation
      connect(chamberPartialA.flange_b,Piston.flange_a) ;
      connect(chamberPartialA.port_A,port_A1) ;
      connect(chamberPartialB.port_A,port_A2) ;
      connect(flange_a,flange_a) ;
      connect(FixedLeft.flange_b,chamberPartialA.flange_a) ;
      connect(FixedRight.flange_b,chamberPartialB.flange_a) ;
      connect(chamberPartialB.flange_b,Piston.flange_b) ;
      connect(Piston.flange_a,flange_a) ;
      connect(Piston.flange_b,flange_b) ;
    end Cylinder;
    
    model SimpleCylinder
      parameter Real k =1 "Raideur ressort de rappel" ;
      parameter Real d =1 "Amortissement ressort de rappel" ;
      parameter Real PistonAreaLeft = 0.001 "Effective area of left side of piston" ;
      parameter Real MobileMass = 100 "Masse mobile (piston + tige)" ;
      parameter Real Stroke = 0.05 "Course totale" ;
      Modelica.Mechanics.Translational.Fixed fixed1(s0 = -Stroke / 2) ;
      Modelica.Mechanics.Translational.Fixed fixed2(s0 = Stroke / 2) ;
      Coselica.Mechanics.Translational.Components.Mass Piston(L = 0, m = MobileMass) ;
      Hydraulic.Parts.ChamberLeft chamberleft1(PistonArea = PistonAreaLeft) ;
      Modelica.Mechanics.Translational.Interfaces.Flange_b flange_b ;
      Modelica.Mechanics.Translational.SpringDamper springdamper1(c = k, d = d);
      Hydraulic.Connectors.Interfaces.PortA porta1 ;
    equation
      connect(springdamper1.flange_b,fixed1.flange_b) ;
      connect(Piston.flange_b,springdamper1.flange_a) ;
      connect(Piston.flange_b,flange_b) ;
      connect(fixed2.flange_b,chamberleft1.flange_a) ;
      connect(chamberleft1.flange_b,Piston.flange_a) ;
      connect(chamberleft1.port_A,porta1) ;
    end SimpleCylinder;
    
    
    model Reservoir "reservoir avec précharge mais sans resistance"
      extends Hydraulic.Connectors.Interfaces.pOnePort;
      parameter Real ppreload = 0.0 "preload pressure";
    equation
      portA.p = ppreload;
    end Reservoir;
    model Valve33 "Valve with 3 ports and 3 positions"
      parameter Real DPnom = 7000000.0 "Nominal difference of pressure (en Pa)";
      parameter Real Qnom = 1 "Nominal flow rate (en m^3/s)";
      parameter Real Imax = 1 "Maximal input";
      Connectors.Interfaces.PortA port_P "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
      Connectors.Interfaces.PortB port_T "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
      Restrictions.TurbulentControled turbulentControled(DPnom = DPnom, Qnom = Qnom, Imax = Imax);
      Connectors.Interfaces.PortA portA ;
      Restrictions.TurbulentControled turbulentControled1(DPnom = DPnom, Qnom = Qnom, Imax = Imax);
      Modelica.Blocks.Interfaces.RealInput i1 ;
      Modelica.Blocks.Math.Gain gain(k = -1) ;
    equation
      connect(turbulentControled.portB,port_P) ;
      connect(turbulentControled.portA,portA) ;
      connect(portA,portA) ;
      connect(port_T,turbulentControled1.portB) ;
      connect(turbulentControled1.portA,portA) ;
      connect(turbulentControled.i,i1) ;
      connect(gain.y,turbulentControled1.i) ;
      connect(gain.u,i1) ;
    end Valve33;
    
model Valve43 "Valve with 4 ports and 3 positions"
      parameter Real DPnom = 7000000.0 "Nominal difference of pressure (en Pa)";
      parameter Real Qnom = 1 "Nominal flow rate (en m^3/s)";
      parameter Real Imax = 1 "Maximal input";
      Connectors.Interfaces.PortA port_P "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
      Connectors.Interfaces.PortB port_T "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
      Restrictions.TurbulentControled turbulentControled(Qnom = Qnom, Imax = Imax, DPnom = DPnom / 2) ;
      Connectors.Interfaces.PortA portA ;
      Restrictions.TurbulentControled turbulentControled1(Qnom = Qnom, Imax = Imax, DPnom = DPnom / 2) ;
      Modelica.Blocks.Interfaces.RealInput i1 ;
      Modelica.Blocks.Math.Gain gain(k = -1) ;
      Restrictions.TurbulentControled turbulentControled2(Qnom = Qnom, Imax = Imax, DPnom = DPnom / 2) ;
      Restrictions.TurbulentControled turbulentControled3(Qnom = Qnom, Imax = Imax, DPnom = DPnom / 2) ;
      Connectors.Interfaces.PortB portB ;
    equation
      connect(gain.u,i1);
      connect(i1,turbulentControled.i) ;
      connect(gain.y,turbulentControled1.i) ;
      connect(turbulentControled3.i,i1) ;
      connect(turbulentControled2.i,gain.y) ;
      connect(port_P,turbulentControled.portA) ;
      connect(turbulentControled.portB,turbulentControled1.portA) ;
      connect(turbulentControled1.portB,turbulentControled2.portA) ;
      connect(turbulentControled3.portA,turbulentControled2.portB) ;
      connect(turbulentControled3.portB,turbulentControled.portA) ;
      connect(portA,turbulentControled.portB) ;
      connect(port_T,turbulentControled1.portB) ;
      connect(portB,turbulentControled2.portB) ;
    end Valve43;
    
    model Valve22 "Valve with 2 ports and 2 positions"
      parameter Real DPnom = 7000000.0 "Nominal difference of pressure (en Pa)";
      parameter Real Qnom = 1 "Nominal flow rate (en m^3/s)";
      parameter Real Imax = 1 "Maximal input";
      Modelica.Blocks.Math.Gain gain(k = -1) ;
      Modelica.Blocks.Interfaces.RealInput i1 ;
      Restrictions.TurbulentControled turbulentControled(DPnom = DPnom, Qnom = Qnom, Imax = Imax) ;
      Connectors.Interfaces.PortA portA ;
      Connectors.Interfaces.PortB portB ;
    equation
      connect(gain.u,i1);
      connect(turbulentControled.i,i1) ;
      connect(turbulentControled.portB,portB) ;
      connect(turbulentControled.portA,portA) ;
    end Valve22;

model Valve53 "Valve with 5 ports and 3 positions"
  parameter Real DPnom = 7000000.0 "Nominal difference of pressure (en Pa)";
  parameter Real Qnom = 1 "Nominal flow rate (en m^3/s)";
  parameter Real Imax = 1 "Maximal input";
  Restrictions.TurbulentControled turbulentControled(Qnom = Qnom, Imax = Imax, DPnom = DPnom / 2) ;
  Restrictions.TurbulentControled turbulentControled1(Qnom = Qnom, Imax = Imax, DPnom = DPnom / 2) ;
  Modelica.Blocks.Interfaces.RealInput i1 ;
  Modelica.Blocks.Math.Gain gain(k = -1) ;
  Restrictions.TurbulentControled turbulentControled2(Qnom = Qnom, Imax = Imax, DPnom = DPnom / 2) ;
  Restrictions.TurbulentControled turbulentControled3(Qnom = Qnom, Imax = Imax, DPnom = DPnom / 2) ;
  Connectors.Interfaces.PortA portA ;
  Connectors.Interfaces.PortB portC ;
  Connectors.Interfaces.PortA port_P "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
  Connectors.Interfaces.PortB port_T "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
  Hydraulic.Connectors.Interfaces.PortB portB ;
equation
  connect(gain.u,i1) ;
  connect(i1,turbulentControled.i) ;
  connect(gain.y,turbulentControled1.i) ;
  connect(turbulentControled3.i,i1) ;
  connect(turbulentControled2.i,gain.y) ;
  connect(port_P,turbulentControled.portA) ;
  connect(turbulentControled.portA,turbulentControled1.portA) ;
  connect(turbulentControled.portB,turbulentControled2.portB) ;
  connect(turbulentControled2.portA,turbulentControled3.portA) ;
  connect(portA,turbulentControled1.portB) ;
  connect(port_T,turbulentControled3.portA) ;
  connect(portB,turbulentControled2.portB) ;
  connect(portC,turbulentControled3.portB) ;
end Valve53;


    
    model PressureForceTranslator "Equals pressure and force for a given area (parameter)"
      parameter Real Area=1;
      Connectors.Interfaces.PortA port ;
      Modelica.Mechanics.Translational.Interfaces.Flange_a flange ;
    equation
      port.p = flange.f / Area;
    end PressureForceTranslator;
    model Pump "Convertisseur Hydraulique / Mecanique rotation parfait"
      parameter Real Cyl = 1 "Volume de la cylindrée (m^3/tour)";
      Real dp "pressure drop";
      Real q "débit dans le composant";
      Real w "angular velocity of pump shaft";
      Connectors.Interfaces.PortA portA "Port A, were oil flows into the component (positive q, port_A.p > port_B.p means positive dp)" ;
      Connectors.Interfaces.PortB portB "Port B, were oil leaves the component (negative q, port_A.p > port_B.p means positive dp)" ;
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange ;
    equation
      dp = portA.p - portB.p;
      q = portA.q;
      portA.q + portB.q = 0;
      w = der(flange.phi);
      // Pompe parfaite
      q = (Cyl * w) / 2 / Coselica.Constants.pi;
      dp = (flange.tau * 2 * Coselica.Constants.pi) / Cyl;
    end Pump;
    
  end Components;
end Hydraulic;

